#####################################################################
# EJ(20170302)
# MED indicators
#####################################################################

library(ggplot2)
library(lme4)
library(influence.ME)
library(lattice)
library(parallel)
source("funs.R")
theme_set(theme_bw())
options(stringsAsFactors=FALSE)
th <- theme(axis.text.x  = element_text(angle=90, vjust=0.5), panel.grid.minor = element_blank())
it <- 500
nc <- 8
# to control de seed in mclapply
RNGkind("L'Ecuyer-CMRG")

#====================================================================
# Load data, subset and transform
#====================================================================

#--------------------------------------------------------------------
# load
#--------------------------------------------------------------------
# Sampling frame
sfm <- read.csv("MED_SF_09.02.2017.csv")
sfm$stk <- tolower(paste(sfm$X3A_CODE, sfm$GSA, sep="_"))

# GFCM
msa <- read.csv("2017-GFCMstockSumm.csv")
msa <- transform(msa, 
	Area = gsub(",", "_", GSA),
	Fref = ifelse(is.na(Fmsy), F01, Fmsy),
	Bref = NA,
	R = NA,
	Stock = substr(stock, 1, 3),
	asses_year = report.year,
	SSB=ssb,
	Meeting="GFCM",
	Assessment_URL=Link.to.pdf,
	Species=species
)
	
msa <- msa[,c("Stock", "Area", "year", "R", "SSB", "F", "Fref", "Blim", "Bref", "asses_year", "Meeting", "Assessment_URL", "Species")]

# STECF
load("cfp2017.RData")
cfp2017$source <- rep("STECF", length(cfp2017$year))

# both
msa <- rbind(cfp2017[,c("Stock", "Area", "year", "R", "SSB", "F", "Fref", "Blim", "Bref", "asses_year", "Meeting", "Assessment_URL", "Species")], msa)
msa[msa$Meeting!="GFCM","Meeting"] <- "STECF"
names(msa)
names(msa)[names(msa)=="Meeting"] <- "source"

#--------------------------------------------------------------------
# transform
#--------------------------------------------------------------------
msa$stk <- tolower(paste(msa$Stock, msa$Area, sep="_"))
msa$StockDescription <- paste(msa$Species, "in GSA", gsub("_", ", ",  msa$Area))
msa <- transform(msa, indF = F/Fref)
msa <- transform(msa, sfFind=!is.na(indF))

#--------------------------------------------------------------------
# subset
#--------------------------------------------------------------------
# filtering through the sampling frame
v0 <- unique(msa$stk)
lst <- strsplit(v0, "_")
lst <- lapply(lst, function(x) paste(x[1], as.numeric(x[-1]), sep="_")) # need the numeric coerce to get rid of leading zero
df0 <- data.frame(msastk=rep(v0, unlist(lapply(lst, length))), stk=do.call("c", lst))
stkToRetain <- unique(df0[df0$stk %in% sfm$stk, "msastk"])
stkToDrop <- v0[!(v0 %in% stkToRetain)]
write.csv(stkToDrop, file="stkToDropBySampFrame-med.csv")
msa <- subset(msa, stk %in% stkToRetain)

# keep last three years of assessments
sam <- msa[!is.na(msa$indF) & msa$year >=2003 & msa$asses_year %in% c("2016", "2015", "2014"),]

#--------------------------------------------------------------------
# remove old assessments and single GSA assessment when there's combined one
#--------------------------------------------------------------------
# drop ARA in 6 2014, there's a more recent assessment
sam <- subset(sam, !(stk=="ara_6" & asses_year==2014))
# drop TUR in 29 2014, there's a more recent assessment
sam <- subset(sam, !(stk=="tur_29" & asses_year==2014))
# drop DPS and HKE in 09 and 07, there are combined assessments
sam <- subset(sam, !(stk %in% c("dps_9", "hke_9", "hke_7")))
# check that stocks with more than one assessment are not in the table
# must return FALSE
sum(apply(table(sam[,c("stk","asses_year")]), 1, function(x) sum(x>0))>1)>0

#--------------------------------------------------------------------
# remove duplicated assessments in the same year
#--------------------------------------------------------------------
# drop STECF assessment of sprat in 29
sam <- subset(sam, !(stk=="spr_29" & source=="STECF"))
# check that stocks assessed by STECF and GFCM in the same year are not in the table
# must return FALSE
sum(apply(table(sam[,c("stk","year")]), 1, ">", 1))>0

#====================================================================
# Indicators
#====================================================================
v0 <- 2003:2015
v0[seq(2,12,2)] <- ""
sc <- scale_x_continuous(breaks=2003:2015, labels=as.character(v0))

#--------------------------------------------------------------------
# Number of stocks
#--------------------------------------------------------------------
mnStks <- aggregate(stk~year, sam, length)
names(mnStks) <- c('year', 'N')

# plot
png("figMedI7a.png", 600, 400)
ggplot(subset(mnStks, year!=2015), aes(x=year, y=N)) + 
	geom_line() + 
	ylab("No. of stocks") + 
	xlab("") + 
	ylim(c(0,50)) + 
	sc + 
	th +
    geom_point(aes(x=2015, y=mnStks$N[length(mnStks$N)]), size=2)
dev.off()

png("figMedI7b.png", 600, 800)
ggplot(sam, aes(year,stk)) + 
	geom_line() +
	ylab("Stock") +
	xlab("Year") +
	sc +
	th +
	geom_vline(xintercept = 2014, col = "red")
dev.off()

#--------------------------------------------------------------------
# F/Fmsy model based indicator
#--------------------------------------------------------------------
v0 <- 2003:2014
v0[seq(2,10,2)] <- ""
sc <- scale_x_continuous(breaks=2003:2014, labels=as.character(v0))
idx <- sam$year!=2015
df0 <- sam[idx,]
df0$year <- factor(df0$year)
yrs <- levels(df0$year)
nd <- data.frame(year=factor(yrs))

# model
mfit <- glmer(indF ~ year + (1|stk), data = df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
runDiagsME(mfit, "stk", df0, "diagMedI5.pdf", nc, nd)

# bootstrap
set.seed(1234)
stk <- unique(df0$stk)
mfit.bs <- split(1:it, 1:it) 
mfit.bs <- mclapply(mfit.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, stk==i))
	fit <- glmer(indF ~ year + (1|stk), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

mfitm <- do.call("rbind", mfit.bs)
mfitq <- apply(mfitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
mfitq <- cbind(year=as.numeric(yrs), as.data.frame(t(mfitq)))

# plot
png("figMedI5mod.png", 600, 400)
ggplot(mfitq, aes(x=year)) + 
	geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) + 
	geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) + 
	geom_line(aes(y=`50%`)) + 
	expand_limits(y=0) + 
	geom_point(aes(x=year[length(year)], y=`50%`[length(`50%`)]), size=2) + 
	geom_hline(yintercept = 1, linetype=2) + 
	ylab(expression(F/F[MSY])) + 
	xlab("") + 
	theme(legend.position = "none") + 
	sc + 
	th
dev.off()

# table
df0 <- t(mfitq)[-1,]
colnames(df0) <- mfitq[,1]
write.csv(df0, file="tabMedI5mod.csv")

#--------------------------------------------------------------------
# SSB indicator
#--------------------------------------------------------------------
# model
samb <- msa[!is.na(msa$SSB) & msa$year >=2003 & msa$asses_year %in% c("2016", "2015", "2014"),]
idx <- samb$year!=2015
df0 <- samb[idx,]
df0$year <- factor(df0$year)
yrs <- levels(df0$year)
nd <- data.frame(year=factor(yrs))

# model
mfitb <- glmer(SSB ~ factor(year) + (1|stk), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
runDiagsME(mfitb, "stk", df0, "diagMedI6.pdf", nc, nd)

# bootstrap
set.seed(1234)
stk <- unique(df0$stk)
mfitb.bs <- split(1:it, 1:it) 
mfitb.bs <- mclapply(mfitb.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, stk==i))
	fit <- glmer(SSB ~ year + (1|stk), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

mfitm <- do.call("rbind", mfitb.bs)
mfitm <- exp(log(mfitm)-mean(log(mfitm[,1]), na.rm=TRUE))
mfitq <- apply(mfitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
mfitq <- cbind(year=as.numeric(yrs), as.data.frame(t(mfitq)))

# plot
png("figMedI6.png", 600, 400)
ggplot(mfitq, aes(x=year)) + 
	geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) + 
	geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) + 
	geom_line(aes(y=`50%`)) + 
	expand_limits(y=0) + 
	geom_point(aes(x=year[length(year)], y=`50%`[length(`50%`)]), size=2) + 
	geom_hline(yintercept = 1, linetype=2) +
	ylab(expression(B/B[2013])) + 
	xlab("") + 
	theme(legend.position = "none") + 
	sc + 
	th
dev.off()

df0 <- t(mfitq)[-1,]
colnames(df0) <- mfitq[,1]
write.csv(df0, file="tabMedI6.csv")

write.csv(sam, file="sam.csv")
save.image("RData.med")


